# et - the trivial time tracker
# Copyright 2014 Ethan Blanton
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

class ETDuration
    attr_reader :seconds

    def initialize(seconds)
        @seconds = seconds.to_i
    end

    def hours
        @seconds / 3600
    end

    def minutes
        (@seconds / 60) % 60
    end

    def self.parse(s)
        case s
        when /^([0-9]+)([smhd])$/i
            case $2
            when 's'
                return ETDuration.new($1.to_i)
            when 'm'
                return ETDuration.new($1.to_i * 60)
            when 'h'
                return ETDuration.new($1.to_i * 3600)
            when 'd'
                return ETDuration.new($1.to_i * 3600 * 24)
            end
        when /^([0-9]+):([0-5][0-9])$/
            return ETDuration.new($1.to_i * 60 + $2.to_i)
        when /^([0-9]+)m([0-9]|[0-5][0-9])s$/
            return ETDuration.new($1.to_i * 60 + $2.to_i)
        when /^([0-9]+)$/
            return ETDuration.new($1.to_i)
        else
            raise ArgumentError.new("Cannot parse #{s}")
        end
    end

    def to_s
        sprintf "%d:%02d", hours, minutes
    end
end
