# et - the trivial time tracker
# Copyright 2014 Ethan Blanton
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

require 'etduration'

class ETReport
    def initialize(tasks, options = {})
        @tasktimes = Hash.new { |h,k| h[k] = 0 }
        @tasktotals = Hash.new { |h,k| h[k] = 0 }
        @tasks = tasks
        @options = options
    end

    def add_task(task, time)
        @tasktimes[task] = time
        @tasktotals[task] += time
        parent = task.parent
        while parent
            @tasktotals[parent] += time
            if parent == @tasks
                break
            end
            parent = parent.parent
        end
    end

    def total
        return ETDuration.new(@tasktotals[@tasks])
    end

    def taskid(task, topdepth = 0)
        if @options[:full_taskid]
            task.full_id
        else
            "#{"  " * (task.depth - topdepth)}#{task.tid}"
        end
    end

    def csv(date)
        lines = []
        @tasks.each do |t|
            next if @tasktimes[t] == 0
            this = ETDuration.new(@tasktimes[t])
            lines << sprintf('%s,%s,%s', date, taskid(t), this)
        end
        lines.join "\n"
    end

    def summary
        topdepth = @tasks.depth == 0 ? 1 : @tasks.depth
        lines = [sprintf('%-20s %-36s %8s %8s', 'Task', 'Description',
                         'Total', 'This')]
        lines << ''
        @tasks.each do |t|
            next if t.depth == 0 or @tasktotals[t] == 0
            this = ETDuration.new(@tasktimes[t])
            total = ETDuration.new(@tasktotals[t])
            lines << sprintf('%-20s %-36s %8s %8s',
                             taskid(t, topdepth),
                             t.description,
                             @tasktotals[t] > @tasktimes[t] ? total : '',
                             @tasktimes[t] > 0 ? this : '')
        end
        lines << ""
        lines << sprintf('%-56s  %8s', 'TOTAL', ETDuration.new(@tasktotals[@tasks]))
        lines.join "\n"
    end

    def to_s
        summary
    end
end
