require 'test/unit'

require 'fileutils'
require 'tmpdir'
require 'json'

require 'etdb'
require 'ettest'

#
# This test suite is WILDLY incomplete
#
class ETdb_Sync_Test < Test::Unit::TestCase
    include ETTest

    def setup
        @tmpdir = Pathname.new(Dir.mktmpdir)
        @dbdir = @tmpdir + 'etdb'
    end

    def teardown
        FileUtils.remove_entry @tmpdir
    end

    def init_db_tree(db)
        ETTask.parse(@@TaskTree).each do |task|
            newtask = ETTask.new(task.tid, task.description, nil, [],
                                 false, task.UUID)
            parent = if task.parent
                         db.find_task "uuid:#{task.parent.UUID}"
                     else
                         nil
                     end
            db.add_task(newtask, parent)
        end
        tasks = 0
        db.tasks.each do |task|
            task.each do |subtask|
                tasks += 1
                assert(subtask.UUID.to_i == tasks)
            end
        end
        assert(tasks == 5)
    end

    def test_dump
        db = ETdb.new @dbdir
        init_db_tree(db)
        othertop = ETTask.new('OTHERTOP', 'Other top-level task', nil, [],
                              false, '6')
        db.add_task(othertop)

        db.checkin(db.find_task(ETTask.idchain('ROOT', 'C1', 'G2')))
        db.bump(-60)
        db.checkout

        File.open(@tmpdir + 'dumpfile', "w") do |f|
            db.dump(nil, f)
        end
        File.open(@tmpdir + 'dumpfile', 'r') do |f|
            assert(f.gets == "et dump version 1.0\n")
            assert(f.gets.to_i == 2)
            root = ETTask.parse(JSON.parse(f.gets))
            checktasks(root, ETTask.parse(@@TaskTree))
            othertop2 = ETTask.parse(JSON.parse(f.gets))
            checktasks(othertop2, ETTask.parse(othertop.to_hash))
            task = nil
            checkins = 0
            f.each do |line|
                case line.rstrip
                when /^task ([0-9a-fA-F-]+)$/
                    task = db.find_task("uuid:#{$1}")
                    assert(task)
                else
                    assert(task.tid == 'G2')
                    checkins += 1
                end
            end
            assert(checkins == 1)
        end
    end

    def test_parse
        db = ETdb.new @dbdir
        db2 = ETdb.new(@tmpdir + 'etdb-new')
        init_db_tree(db)
        othertop = ETTask.new('OTHERTOP', 'Other top level task', nil, [],
                              false, '6')
        db.add_task(othertop)
        db.checkin(db.find_task('OTHERTOP'))
        db.bump(-120)
        db.checkout
        db.checkin(db.find_task(ETTask.idchain('ROOT', 'C1')))
        db.bump(-60)
        db.checkout
        File.open(@tmpdir + 'dumpfile', 'w') do |f|
            db.dump(db2.UUID, f)
        end

        File.open(@tmpdir + 'dumpfile', 'r') do |f|
            db2.process_dump(f)
        end
        tasks = 0
        db2.tasks.each do |task|
            task.each do |subtask|
                tasks += 1
                assert(subtask.UUID.to_i == tasks)
            end
        end
        assert(tasks == 6)
        File.open(db2.logdir + db2.find_task('OTHERTOP').UUID, 'r') do |f|
            lines = 0
            f.each do
                lines += 1
            end
            assert(lines == 1)
        end
        File.open(db2.logdir + db2.find_task(ETTask.idchain('ROOT', 'C1')).UUID, 'r') do |f|
            lines = 0
            f.each do
                lines += 1
            end
            assert(lines == 1)
        end

        db.checkin(db.find_task('OTHERTOP'))
        db.checkout
        db.move_task(db.find_task('OTHERTOP'), ETTask.idchain('ROOT', 'C2', 'G3'))

        File.open(@tmpdir + 'dumpfile', 'w') do |f|
            db.dump db2.UUID, f
        end

        File.open(@tmpdir + 'dumpfile', 'r') do |f|
            db2.process_dump f
        end

        g3 = db2.find_task(ETTask.idchain('ROOT', 'C2', 'G3'))
        assert(g3)
        assert(g3.UUID == '6')

        File.open(db2.logdir + '6', 'r') do |f|
            lines = 0
            f.each do
                lines += 1
            end
            assert(lines == 2)
        end
    end

    def test_rearrange
        db = ETdb.new(@dbdir)
        db2 = ETdb.new(@tmpdir + 'etdb-new')
        t1 = ETTask.new('TASK', 'Original root task')
        t2 = ETTask.new('SUBTASK', 'Test subtask')
        t3 = ETTask.new('NEWROOT', 'New root task')

        db.add_task(t1)
        db.add_task(t2, t1)

        File.open(@tmpdir + 'dumpfile', 'w') do |f|
            db.dump(nil, f)
        end
        File.open(@tmpdir + 'dumpfile', 'r') do |f|
            db2.process_dump(f)
        end

        db.add_task(t3)
        db.move_task(t2, ETTask.idchain('NEWROOT', 'SUBTASK'))

        File.open(@tmpdir + 'dumpfile', 'w') do |f|
            db.dump(nil, f)
        end
        File.open(@tmpdir + 'dumpfile', 'r') do |f|
            db2.process_dump(f)
        end
    end


end
