require 'test/unit'

require 'ettask'
require 'ettest'

class ETTask_Basic_Test < Test::Unit::TestCase
    include ETTest

    def test_serialize
        task = ETTask.new('ROOT', 'Root task')
        child = ETTask.new('CHILD', 'Child task')
        task.add_subtask(child)
        taskhash = task.to_hash
        task_copy = ETTask.parse(taskhash)
        checktasks(task, task_copy)

        tasks = ETTask.parse(@@TaskTree)
        hash = tasks.to_hash
        tasks_copy = ETTask.parse(@@TaskTree)
        checktasks(tasks, tasks_copy)
    end

    def test_each
        tasks = ETTask.parse(@@TaskTree)
        uuid = 0
        tasks.each do |task|
            assert(task.UUID.to_i == uuid + 1)
            uuid += 1
        end
    end

    def test_depth
        tasks = ETTask.parse(@@TaskTree)
        assert(tasks.depth == 0)
        assert(tasks.subtask('C1').depth == 1)
        assert(tasks.subtask('C1').subtask('G1').depth == 2)
    end

    def test_remove
        tasks = ETTask.parse(@@TaskTree)
        copy = ETTask.parse(@@TaskTree["subtasks"][1])

        caught = false
        begin
            tasks.remove_subtask(copy)
        rescue ArgumentError
            caught = true
        end
        assert(caught)

        task = tasks.subtask('C1')
        tasks.remove_subtask(task)
        assert(tasks.subtasks.length == 1)
        copy = ETTask.parse(@@TaskTree["subtasks"][0])
        checktasks(task, copy)
    end

    def test_rename
        hash = {
            "tid" => "oldtid",
            "description" => "old name",
            "uuid" => "abcdef",
            "inactive" => false,
            "subtasks" => []
        }

        oldtask = ETTask.parse(hash)
        hash["tid"] = "newtid"
        newtask = ETTask.parse(hash)
        oldtask.rename('newtid')
        checktasks(oldtask, newtask)
        hash["description"] = "new name"
        newtask = ETTask.parse(hash)
        oldtask.rename(oldtask.tid, "new name")
        checktasks(oldtask, newtask)

        tasks = ETTask.parse(@@TaskTree)
        c2task = tasks.subtask('C2')
        c2task.rename('C2-new')
        assert(c2task.tid == 'C2-new')
        newtask = tasks.subtask('C2-new')
        checktasks(c2task, newtask)
    end
end
